/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.waiter;
import com.sos.JSHelper.io.Files.JSFile;
import com.sos.i18n.I18NBase;
import com.sos.i18n.annotation.I18NResourceBundle;
import com.sos.jobnet.classes.*;
import com.sos.jobnet.classes.Status.enuStatus;
import com.sos.jobnet.db.*;
import com.sos.jobnet.interfaces.IJobNetCollection;
import com.sos.jobnet.interfaces.IJobNetPlanEventListener;
import com.sos.jobnet.interfaces.IPlanItem;
import com.sos.jobnet.interfaces.IWaitHandler;
import com.sos.jobnet.options.JobNetOptions;
import org.apache.log4j.Logger;

import java.io.File;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
* \class JobNetDBWaiter 
* 
* \brief JobNetDBWaiter - 
* 
* \details
*
* \section JobNetDBWaiter.java_intro_sec Introduction
*
* \section JobNetDBWaiter.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id: JobNetDBWaiter.java 20191 2013-05-17 14:49:02Z ss $
* \see reference
*
* Created on 22.02.2012 13:56:28
 */
/**
 * @author oh
 *
 */
@I18NResourceBundle(baseName = "com_sos_jobnet_messages", defaultLocale = "en")
public class JobNetDBWaiter extends I18NBase implements IWaitHandler {
	
	private final String		conClassName			= "JobNetDBWaiter";
	private	final String		conSVNVersion			= "$Id: JobNetDBWaiter.java 20191 2013-05-17 14:49:02Z ss $";
	private static final Logger	logger					= Logger.getLogger(JobNetDBWaiter.class);
	private List<JobNetEdgesDBItem> jobNetEdgesList		= null;
	private IJobNetCollection 	waitforNodes 			= new JobNetCollection();
	
	private static final String	JOBNETDBW_E_0010		= "JOBNETDBW_E_0010";		// %s: No item found for the current node (%s) in database table SCHEDULER_JOB_NET_PLAN.
	private static final String	JOBNETDBW_E_0011		= "JOBNETDBW_E_0011";		// %s: No predecessor with node_id=%d found in database table SCHEDULER_JOB_NET_PLAN.
	
	private final JobNetOptions options;
	private final String uuidJobnetIdentifier;
	private final String jobNet;
	private final String node;
	private final String schedulerId;
	private final JSFile hibernateConnectionConfigFile;
	private final String curNodeTitle;
	private final List<String> waitForEvents;
	private final JobNetPlanDBItem currentRecord;
	
	private final EventsDBLayer eventsDBLayer;
	private final JobNetPlanDBLayer	jobNetPlanDBLayer;
	private IJobNetPlanEventListener eventListener = null;
	
	public JobNetDBWaiter(JobNetOptions jobNetOptions) {
		super(JobNetConstants.strBundleBaseName);
		this.options = jobNetOptions;
		this.uuidJobnetIdentifier = options.uuid_jobnet_identifier.Value();
		this.jobNet = options.jobnet.Value();
		this.node = options.node.Value();
		this.schedulerId = options.scheduler_id.Value();
		this.hibernateConnectionConfigFile 	= options.hibernate_connection_config_file.JSFile();
 		this.curNodeTitle = uuidJobnetIdentifier + ":" + schedulerId + ":" + jobNet + ":" + node;
		this.waitForEvents = getWaitForEvents(options.wait_for_events.Value());

		String conMethodName = this.getClass().getSimpleName(); 
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "uuid_jobnet_identifier", uuidJobnetIdentifier));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "jobnet", jobNet));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "node", node));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "scheduler_id", schedulerId));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "hibernate_connection_config_file", hibernateConnectionConfigFile.getAbsolutePath()));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "wait_for_events", waitForEvents));

		this.jobNetPlanDBLayer 	= new JobNetPlanDBLayer(hibernateConnectionConfigFile, schedulerId);
		this.eventsDBLayer = new EventsDBLayer(hibernateConnectionConfigFile);
		this.currentRecord = readCurrentRecordFromDBAndLock();
	}
	
	private JobNetPlanDBItem readCurrentRecordFromDBAndLock() {
		
		final String conMethodName = conClassName + "::getCurNode";
		JobNetPlanDBItem result = null;
		JobNetPlanFilter filter = new JobNetPlanFilter();
		filter.setOrderCriteria(null);
		filter.setSchedulerId(schedulerId);
		filter.setUuid(uuidJobnetIdentifier);
		filter.setJobnet(jobNet);
		filter.setNode(node);
		jobNetPlanDBLayer.setFilter(filter);
		
		List<JobNetPlanDBItem> jobNetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanListWithLock(0);
		if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
			String msg = Messages.getMsg(JOBNETDBW_E_0010, conMethodName, curNodeTitle);
			logger.error(msg);
			throw new JobNetException(msg);
		}
		else {
			result = jobNetPlanDBItem.get(0);
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0010, conMethodName, result.getTitle()));
		}
		return result;
		
	}
	
	private List<String> getWaitForEvents(String eventsString) {
		return Arrays.asList(eventsString.split(";"));
	}
	
	/**
	 * \brief getCurNode
	 * 
	 * \details
	 */
	private JobNetPlanDBItem getCurrentRecord() {
		return currentRecord;
	}
		
	/**
	 * \brief getJobnetEdgesList
	 * 
	 * \details
	 */
	private List<JobNetEdgesDBItem> getJobnetEdgesList() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getJobnetEdgesList";
		getCurrentRecord();
		if (jobNetEdgesList == null) {
			File configurationFile 	= new File(hibernateConnectionConfigFile.getAbsolutePath());
			JobNetEdgesDBLayer jobNetEdgesDBLayer = new JobNetEdgesDBLayer(configurationFile);
			jobNetEdgesList = jobNetEdgesDBLayer.getPredecessors(getCurrentRecord().getJobnetNodeDBItem());
		}
		return jobNetEdgesList;
	} // public List<JobNetEdgesDBItem> getJobnetEdgesList
	
	
	/**
	 * \brief updateWaiterStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateWaiterStatus(enuStatus status) {
		final String conMethodName = conClassName + "::updateWaiterStatus";
		JobNetPlanDBItem record = getCurrentRecord();
		if (record != null) {
			int oldState = record.getStatusWaiter();
			record.setStatusWaiter(status.value());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0010), Messages.getMsg(status.msgKey())));
			update();
			if (eventListener != null)
				eventListener.waiterStatusChanged(status, enuStatus.fromInt(oldState) );
//			after each updateWaiterStatus follows updateNodeStatus which calls the commit, 
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(new Date());
		}
	} // private void updateWaiterStatus
	
	public Status getWaiterStatus() {
		getCurrentRecord();
		Status status = new Status();
		return status;
	}
	
	/**
	 * \brief updateRunnerStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateRunnerStatus(enuStatus status) {
		final String conMethodName = conClassName + "::updateRunnerStatus";
		JobNetPlanDBItem record = getCurrentRecord();
		if (record != null) {
			int oldState = record.getStatusRunner();
			record.setStatusRunner(status.value());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), Messages.getMsg(status.msgKey())));
			update();
			if (eventListener != null)
				eventListener.runnerStatusChanged(status, enuStatus.fromInt(oldState) );
//			after each updateRunnerStatus follows updateNodeStatus which calls the commit, 
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(new Date()); 
		}
	} // private void updateRunnerStatus
	
	
	/**
	 * \brief updateNodeStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateNodeStatus(enuStatus status) {
		final String conMethodName = conClassName + "::updateNodeStatus";
		JobNetPlanDBItem rec = getCurrentRecord();
		if (rec!=null) {
			int oldState = rec.getStatusNode();
			rec.setStatusNode(status.value());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0013), Messages.getMsg(status.msgKey())));
			update();
			nodeStatusChanged(rec, enuStatus.fromInt(oldState) );
		}
	} // private void updateNodeStatus
	
	
	/**
	 * \brief update
	 * 
	 * \details
	 */
	private void update() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::update";
		getCurrentRecord().setModified( new Date() );
		getCurrentRecord().setModifiedBy(JobNetDBWaiter.class.getName());
		jobNetPlanDBLayer.beginTransaction();
		jobNetPlanDBLayer.saveOrUpdate(getCurrentRecord());
		jobNetPlanDBLayer.commit();
	} // private void update
	
	
	private String getI18NYesNo(boolean flgIsYes) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getI18NYesNo";
		String msg = Messages.getMsg(JobNetConstants.JOBNET_T_0002); //No
		if (flgIsYes) {
			msg = Messages.getMsg(JobNetConstants.JOBNET_T_0001); //Yes
		}
		return msg;
	}

	
	/**
	 * \brief isBootstrap
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isBootstrap() {
		final String conMethodName = conClassName + "::isBootstrap";
		boolean isBootstrap = getCurrentRecord().getBootstrap();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0021, conMethodName, getI18NYesNo(isBootstrap)));
		return isBootstrap;
	} // public boolean isBootstrap

	
	/**
	 * \brief setWaiterStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToRunning() {
		updateWaiterStatus(enuStatus.RUNNING);
	}

	/**
	 * \brief setNodeStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToRunning() {
		updateNodeStatus(enuStatus.RUNNING);
	}
	


	/**
	 * \brief isWaiterSkipped
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isWaiterSkipped() {
		final String conMethodName = conClassName + "::isWaiterSkipped";
		boolean isSkipped = getCurrentRecord().getIsWaiterSkipped();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0020, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0010), getI18NYesNo(isSkipped)));
		return isSkipped;
	}
	

	/**
	 * \brief isRunnerSkipped
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isRunnerSkipped() {
		final String conMethodName = conClassName + "::isRunnerSkipped";
		boolean isSkipped = getCurrentRecord().getIsRunnerSkipped();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0020, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), getI18NYesNo(isSkipped)));
		return isSkipped;
	}
	
	@Override
	public boolean isRunnerRunning() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isRunnerRunning";
		boolean isRunning = new Status().isRunning(getCurrentRecord().getStatusRunner());
		return isRunning;
	}
	
	/**
	 * \brief isRunnerOnDemand
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isRunnerOnDemand() {
		final String conMethodName = conClassName + "::isRunnerOnDemand";
		boolean isOnDemand = getCurrentRecord().getIsRunnerOnDemand();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0022, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), getI18NYesNo(isOnDemand)));
		return isOnDemand;
	}

	/**
	 * \brief setNodeStatusToWaiting
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToWaiting() {
		updateNodeStatus(enuStatus.WAITING);
	}

	/**
	 * \brief setRunnerStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setRunnerStatusToRunning() {
		updateRunnerStatus(enuStatus.RUNNING);
	}

	/**
	 * \brief setWaiterStatusToFinish
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToFinish() {
		updateWaiterStatus(enuStatus.FINISHED);
	}

	/**
	 * \brief setWaiterStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToError() {
		updateWaiterStatus(enuStatus.ERROR);
	}

	/**
	 * \brief setNodeStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToError() {
		updateNodeStatus(enuStatus.ERROR);
	}
	
	/**
	 * \brief setStartTime
	 * 
	 * \details
	 */
	@Override
	public void setStartTime() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setStartTime";
		if (getCurrentRecord() != null) {
			Date now = new Date();
			getCurrentRecord().setStartTime(now);
//			after each setStartTime follows updateNodeStatus which calls the commit, 
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(now); 
		}
	}
	
	@Override
	public IJobNetCollection getWaitingNodes() {
		final String conMethodName = conClassName + "::getWaitingNodes";
		getJobnetEdgesList();
		
		int status = -1;
		Status objStatus = new Status();
		List<JobNetPlanDBItem> jobNetPlanDBItem = null;
		JobNetPlanDBItem jobnetPlanDBItem = null;
		waitforNodes = new JobNetCollection();
		
		for (JobNetEdgesDBItem jobnetEdgesItem : jobNetEdgesList) {
			jobNetPlanDBLayer.getFilter().setNode(null);
			jobNetPlanDBLayer.getFilter().setNodeId(jobnetEdgesItem.getChildNodeId());
			jobNetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanList(0);
			
			if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
				String msg = Messages.getMsg(JOBNETDBW_E_0011, conMethodName, jobnetEdgesItem.getChildNodeId()); 
				logger.error(msg);
				throw new JobNetException(msg);
			}
			
			jobnetPlanDBItem = jobNetPlanDBItem.get(0);
			String predecessor = jobnetPlanDBItem.getJobnetNodeItem().getNode();
			status = jobnetPlanDBItem.getStatusNode();
			
			logger.info(Messages.getMsg(JobNetConstants.JOBNETW_I_0010, conMethodName, predecessor, objStatus.getMsg(status)));
			if (objStatus.isFinished(status)) {
				continue;
			}
			waitforNodes.add(jobnetPlanDBItem);
		}
		return waitforNodes;
	}

	@Override
	public void open() {
		logger.info(String.format("%1s %2s", conClassName, conSVNVersion));
		getCurrentRecord();
		//TODO skips in Options lesen und in DB setzen
		//Ist unnoetig, wenn das dashboard die Skip-Funktionalitaet zur Verfuegung stellt
	}

	@Override
	public void close() {
		eventsDBLayer.closeSession();
		jobNetPlanDBLayer.closeSession();
	}

	@Override
	public EventsCollection getMissingEvents() {
		return eventsDBLayer.checkEvents(waitForEvents);
	}

	@Override
	public boolean isNodeStarted() {
		int status = getCurrentRecord().getStatusNode();
		return (enuStatus.fromInt(status) == enuStatus.STARTED);
	}

	@Override
	public boolean isNodeStarting() {
		int status = getCurrentRecord().getStatusNode();
		return (enuStatus.fromInt(status) == enuStatus.STARTING);
	}

	@Override
	public boolean isNodeNotProcessed() {
		int status = getCurrentRecord().getStatusNode();
		return (enuStatus.fromInt(status) == enuStatus.NOT_PROCESSED);
	}


	public void nodeStatusChanged(IPlanItem node, enuStatus oldState) {
		if(eventListener != null) {
			eventListener.nodeStatusChanged(node, oldState);
		}
	}
	
	@Override
	public boolean isReadyForProcessing() {
		return ( isNodeStarted() || isNodeInError() || isJobnetStart() );
	}
	
	@Override
	public boolean isJobnetStart() {
		return (isNodeNotProcessed() && isBootstrap());
	}

	public void setNodeStatusListener(IJobNetPlanEventListener listener) {
		this.eventListener = listener;
	}

	@Override
	public boolean isNodeInError() {
		int status = getCurrentRecord().getStatusNode();
		return (enuStatus.fromInt(status) == enuStatus.ERROR);
	}
	
	@Override
	public void createEvent(EventsDBItem record) {
		eventsDBLayer.createEvent(record);
	}
	
}
