/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "spooler.h"

namespace sos {
namespace scheduler {

//--------------------------------------------------------------------------------------------const

const int  default_kbyte_of_db_lob_entry           = INT_MAX;

//-------------------------------------------------------------------------------Settings::Settings

Settings::Settings()
: 
    _zero_(this+1) 
{}

//------------------------------------------------------------------------------Settings::~Settings

Settings::~Settings() {}

//---------------------------------------------------------------------------Settings::set_defaults

void Settings::set_defaults(Spooler* spooler) {
    if (_html_dir.empty())  
        _html_dir = spooler->home_directory() + "/operations_gui";                  
}

//---------------------------------------------------------------------------Settings::set_defaults

void Settings::set_from_variables(const Com_variable_set& p) {
    _keep_order_content_on_reschedule = p.get_bool("scheduler.order.keep_order_content_on_reschedule", true);
    _max_length_of_blob_entry = p.get_int("scheduler.max_kbyte_of_db_log_entry", default_kbyte_of_db_lob_entry) * 1024;
    _order_distributed_balanced = p.get_bool("scheduler.order.distributed.balanced", false);
}

//------------------------------------------------------------------------------------Settings::set

void Settings::set(int number, const string& value) {
    // number wird in Java SettingName festgelegt.

    switch (number) {
        case 1: 
            _db_name = value; 
            break;
        case 2: 
            _job_java_classpath = value; 
            break;
        case 3:
            _html_dir = value;
            break;
        case 4:
            _job_java_options = value;
            break;
        case 5:
            _use_java_persistence = as_bool(value);
            break;
        case 7:
            _order_distributed_balanced = as_bool(value);
            break;
        default:
            z::throw_xc("UNKNOWN_SETTING", number);
    }
}

//------------------------------------------------------------------------------------Settings::get

string Settings::get(int number) const {
    // number wird in Java SettingName festgelegt.

    switch (number) {
        case 1: return _db_name;
        case 2: return _job_java_classpath;
        case 3: return _html_dir;
        default:
            z::throw_xc("UNKNOWN_SETTING", number);
    }
}

}} //namespace sos::scheduler
