/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id$

#ifndef __ZSCHIMMER_HAS_JAVA_PROXY_
#define __ZSCHIMMER_HAS_JAVA_PROXY_

#include <typeinfo>
#include "java.h"
#include "Proxy_class.h"

namespace zschimmer {
namespace javabridge {

struct Proxy_class;

//----------------------------------------------------------------------------------------Has_proxy
// Legt bei Bedarf einen CppProxy an (vom Java-C++-Generator erzeugt) und und lst ihn am Ende wieder.
// Der CppProxy hlt nicht dieses C++-Object. Wenn das C++-Objekt zerstrt wird, wird die Verbindung vom
// CppProxy zum C++-Objekt gelst und die nchste Verwendung des CppProxy liefert den Fehler Z-JAVA-111.

struct Has_proxy
{
                                Has_proxy                   (Has_proxy* sister_context_proxy);
    virtual                    ~Has_proxy                   ();

    jobject                     j                           ()                                      { return java_proxy_jobject(); }
    jobject                     java_proxy_jobject          ();
    virtual jobject             java_sister                 (); 

    static jobject              jobject_of                  (Has_proxy* a)                          { return a? a->java_proxy_jobject() : NULL; }

protected:
    virtual const Proxy_class*  proxy_class                 () const                                = 0;

private:
    template<class SELF> friend struct has_proxy;

    jlong                       cpp_reference               ()                                      { return (jlong)this; }
    static Has_proxy*           of_cpp_reference_           (jlong, const char* debug_string);

    void                        cache_proxy_class           ();
    void                    set_reference_in_proxy          (jlong);
    void                        create_proxy                ();

    Proxy_class const*         _proxy_class;
    Global_jobject2            _proxy;
    Has_proxy* const           _sister_context_proxy;       // Kontext fr die Java-Schwester oder NULL
    Global_jobject2            _java_sister;                // Manche C++-Klassen haben neben dem C++-Proxy eine Java-Schwester, eine von C++ unabhngige Klasse
};

//--------------------------------------------------------------------------------------has_proxy<>

template<class SELF>
struct has_proxy : Has_proxy
{
                                has_proxy                   (Has_proxy* sister_context_proxy = NULL) : Has_proxy(sister_context_proxy) {}
    static SELF*                of_cpp_reference            (jlong cpp_reference, const char* debug_string) { return (SELF*)(has_proxy<SELF>*)of_cpp_reference_(cpp_reference, debug_string); }
    static void                 register_cpp_proxy_class_in_java();

    static const class_factory<Proxy_class> proxy_class_factory;

  protected:
    const Proxy_class*          proxy_class                 () const                                { return (const Proxy_class*)proxy_class_factory.clas(); }
};

//----------------------------------------------------------------------------------proxy_jobject<>

struct Proxy_jobject {
  protected:
                                Proxy_jobject               (jobject jo = NULL)                     : _jobject(jo) {}

  public:
    jobject                     get_jobject                 () const                                { return _jobject; }
    void                        set_jobject                 (jobject jo)                            { _jobject = jo; }

  private:
    jobject                    _jobject;
};

//----------------------------------------------------------------------------------proxy_jobject<>
// Typsicheres jobject, fr Parameterbergabe von C++ an Java, 
// damit nicht Global_jobject2 mit NewGlobalRef() und DeleteGlobalRef() verwendet werden muss.

template<class PROXY>   // PROXY: Vom C++/Java-Generator erzeugte Klasse fr Java-Proxy, z.B. SpoolerC
struct proxy_jobject : Proxy_jobject
{
                                proxy_jobject               (jobject jo = NULL)                     : Proxy_jobject(jo) { if (jo) PROXY::java_class_()->assert_is_assignable_from(jo); }
};

//--------------------------------------------------------------------------------java_array_from_c

jobjectArray java_array_from_c(const std::vector<string>&);
jbyteArray java_byte_array_from_c(const string&);
string string_from_java_byte_array(jbyteArray);

//Fr Has_proxy nicht getestet:
//template<class T>
//jobjectArray java_array_from_c(const std::vector<T>& v) {
//    JNIEnv* jenv = jni_env();
//    Local_frame java_frame (10);
//    jobjectArray result;
//    result = jenv->NewObjectArray(v.size(), Vm::static_vm->standard_classes()->_java_lang_object_class, NULL);
//    if (result == NULL) return NULL;
//    for (int i = 0; i < v.size(); i++)
//        jenv->SetObjectArrayElement(result, i, v[i]->java_proxy_jobject());
//    return result;
//}

} //namespace javabridge
} //namespace zschimmer

#endif
