/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: Memory_allocator.cxx 14149 2010-12-09 15:10:35Z jz $

#include "zschimmer.h"
#include "Memory_allocator.h"

using namespace std;

namespace zschimmer {

//-------------------------------------------------------------------------------------------static

Mutex                  Memory_allocator::_mutex;
Memory_allocator::Map  Memory_allocator::_map;
bool Memory_allocator::static_initialized = false;
bool Memory_allocator::static_debug_memory = false;
const char* environment_variable_name = "SOS_SCHEDULER_DEBUG_MEMORY";

/////////////////////////////////////////
// Strzt am Ende ab, weil Memory_allocator::_mutex und ::_map zerstrt sind, wenn noch ein z_free() aufgerufen wird.
// Also nur zum Debuggen verwenden.
/////////////////////////////////////////

//-----------------------------------------------------------------------------------------z_malloc

void* z_malloc(size_t size, const char* name, int lineno) 
{ 
    return Memory_allocator::debug_memory()? Memory_allocator::z_malloc(size, name, lineno) 
                                           : malloc(size);
}

//-------------------------------------------------------------------------------------------z_free

void z_free(void* p)
{ 
    assert(Memory_allocator::static_initialized);
    return Memory_allocator::debug_memory()? Memory_allocator::z_free(p) 
                                           : free(p);
}

//-------------------------------------------------------------------------------------------------

#if defined Z_WINDOWS  &&  defined _DEBUG

    inline void*                my_malloc               ( size_t size, const char* name, int lineno ) { return _malloc_dbg( size, _CLIENT_BLOCK, name, lineno ); }
    inline void                 my_free                 ( void* p )                                   { return _free_dbg( p, _CLIENT_BLOCK ); }

#else

    inline void*                my_malloc               ( size_t size, const char*, int )             { return malloc( size ); }
    inline void                 my_free                 ( void* p )                                   { return free( p ); }

#endif

//-----------------------------------------------------------Memory_allocator::debug_memory

bool Memory_allocator::debug_memory() {
    if (!static_initialized) {
        static_debug_memory = getenv(environment_variable_name) != NULL;
        static_initialized = true;
    }

    return static_debug_memory;
}

//-----------------------------------------------------------------------Memory_allocator::z_malloc
    
void* Memory_allocator::z_malloc( size_t size, const char* name, int lineno )
{
    void* result = NULL;

    Z_FAST_MUTEX(_mutex) {
        result = my_malloc(size, name, lineno);
        _map.insert(Map::value_type(result, Memory_type(name, lineno, size)));
    }

    return result;
}
 
//-------------------------------------------------------------------------Memory_allocator::z_free

void Memory_allocator::z_free( void* p )
{
    Z_FAST_MUTEX(_mutex) {
        _map.erase(p);
        my_free(p);
    }
}

//-------------------------------------------------------------Memory_allocator::memory_counter_map

Memory_allocator::Allocation_map Memory_allocator::allocation_map()
{
    Allocation_map result;

    Z_FAST_MUTEX(_mutex) {
        Z_FOR_EACH_CONST(Map, _map, it) {
            Allocation& e = result[it->second];
            e._count++;
        }
    }

    return result;
}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
