/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "zschimmer.h"

#include "../javaproxy/java__lang__String.h"
#include "xslt_java.h"
#include "log.h"
#include "file.h"

typedef ::javaproxy::java::lang::String StringJ;

using namespace std;
using namespace zschimmer::com;

namespace zschimmer {
namespace xml {

//--------------------------------------------------------------------------------------------const

static Message_code_text error_codes[] =
{
    { "JAVAXSLT-001", "No stylesheet loaded" },
    { "JAVAXSLT-002", "Not parameter syntax NAME=WERT: $1" },
    { "JAVAXSLT-003", "Stylesheet not loadable" },
    { "JAVAXSLT-004", "Error when applying stylesheet \"$1\": $2" },
    { "JAVAXSLT-005", "Only one sort of quote ['\"] in parameter value is supported: $1=$2" },
    { NULL, NULL }
};

//-------------------------------------------------------------------------------------------Z_INIT

Z_INIT(xml_javaxml) {
    add_message_code_texts( error_codes );
}

//-----------------------------------------------------------------Xslt_parameters::Xslt_parameters

Xslt_parameters::Xslt_parameters(int)
: 
   _hashMapJ(HashMapJ::new_instance())
{}

//----------------------------------------------------------------Xslt_parameters::~Xslt_parameters
    
Xslt_parameters::~Xslt_parameters()
{}

//------------------------------------------------------------------------Xslt_parameters::allocate

void Xslt_parameters::allocate(int)
{           
    _hashMapJ = HashMapJ::new_instance();
}

//-----------------------------------------------------------------------Xslt_parameters::set_xpath

void Xslt_parameters::set_xpath(int, const string& name, const string& value)
{
    _hashMapJ.put(StringJ(name), StringJ(value));
    //Parameter p;
    //p._name = name;
    //p._value = value;
    //_parameters[i] = p;
}

//----------------------------------------------------------------------Xslt_parameters::set_string

void Xslt_parameters::set_string(int i, const string& name, const string& value)
{
    Z_LOG2("zschimmer", Z_FUNCTION << " " << i << " " << name << "=" << value << "\n");

    string string_value;

    if( value.find( '"' ) != string::npos )
    {
        if( value.find( '\'' ) != string::npos )  throw_xc( "JAVAXSLT-005", name, value );    // Anscheinend kennt libxslt keine Ersatzdarstellung fr Anfhrungszeichen
        string_value = "'" + value + "'";
    }
    else
    {
        string_value = "\"" + value + "\"";
    }

    set_xpath( i, name, string_value );
}

//-----------------------------------------------------------------Xslt_stylesheet::Xslt_stylesheet

Xslt_stylesheet::Xslt_stylesheet()
{
}

//-----------------------------------------------------------------Xslt_stylesheet::Xslt_stylesheet

Xslt_stylesheet::Xslt_stylesheet( const string& xml_or_filename )
{
    if( is_xml( xml_or_filename ) )
    {
        load( Document_ptr( xml_or_filename ) );
    }
    else
    {
        load_file( xml_or_filename );
    }
}

//-----------------------------------------------------------------Xslt_stylesheet::Xslt_stylesheet

Xslt_stylesheet::Xslt_stylesheet( const BSTR xml_or_filename_bstr )
{
    if( is_xml( xml_or_filename_bstr ) )
    {
        load( Document_ptr( xml_or_filename_bstr ) );
    }
    else
    {
        load_file( string_from_bstr( xml_or_filename_bstr ) );
    }
}

//----------------------------------------------------------------Xslt_stylesheet::~Xslt_stylesheet

Xslt_stylesheet::~Xslt_stylesheet()
{}

//--------------------------------------------------------------------------Xslt_stylesheet::is_xml

bool Xslt_stylesheet::is_xml( const string& str )
{
    return string_begins_with( str, "<" );
}

//--------------------------------------------------------------------------Xslt_stylesheet::is_xml

bool Xslt_stylesheet::is_xml( const BSTR bstr )
{
    return bstr && bstr[0] == '<';
}

//-----------------------------------------------------------------------Xslt_stylesheet::load_file

void Xslt_stylesheet::load_file( const string& path )
{
    _path = path;   // Auch im Fehlerfall
    _stylesheetJ = CppXsltStylesheetJ::new_instance(Document_ptr(file::string_from_file(path)).ref());
}

//------------------------------------------------------------------------Xslt_stylesheet::load_xml

void Xslt_stylesheet::load( const Document_ptr& stylesheet )
{
    _path = "";
    _stylesheetJ = CppXsltStylesheetJ::new_instance(stylesheet.ref());
}

//---------------------------------------------------------------------------Xslt_stylesheet::apply

Document_ptr Xslt_stylesheet::apply( const Document_ptr& document )
{
    Xslt_parameters no_parameters ( 0 );
    return apply( document, no_parameters );
}

//---------------------------------------------------------------------------Xslt_stylesheet::apply

Document_ptr Xslt_stylesheet::apply( const Document_ptr& document, const Xslt_parameters& parameters )
{
    if(!_stylesheetJ)  throw_xc( "JAVAXSLT-001", _path );

    return _stylesheetJ.apply(document.ref(), parameters._hashMapJ);
}

//-------------------------------------------------------------------------------------------------

} //namespace xml
} //namespace
