/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_gzip.h 13199 2007-12-06 14:15:42Z jz $

#ifndef __ZSCHIMMER_Z_GZIP_H
#define __ZSCHIMMER_Z_GZIP_H

namespace zschimmer {

//-------------------------------------------------------------------------------------------------

string                          string_gzip_deflate     ( const Byte*, size_t length );
inline string                   string_gzip_deflate     ( const char* p, size_t length )            { return string_gzip_deflate( reinterpret_cast<const unsigned char*>( p ), length ); }
inline string                   string_gzip_deflate     ( const string& s )                         { return string_gzip_deflate( s.data(), s.length() ); }

//-------------------------------------------------------------------------------------------------

//void                            gzip_file               ( const File_path& source, const File_path& destination );

//--------------------------------------------------------------------------------Simple_byte_queue

struct Simple_byte_queue
{
                                Simple_byte_queue       ( int buffer_size = default_chunk_size )    : _zero_(this+1), _chunk_size( buffer_size ) {}
                               ~Simple_byte_queue       ()                                          { clear(); }

    void                        clear                   ();

    Byte*                       write_buffer            ( int size )                                { return request_write_buffer_2( size, 0 ); } // 1. Pointer lesen
    int                         write_buffer_size       ();                                                                                // 2. dann Gre des Puffers
    void                        on_write_buffer_written ( int count )                               { _write_position += count; }          // 3. Schlielich die Lnge setzen

    Byte*                       request_write_buffer    ( int length )                              { return request_write_buffer_2( length, length ); }

    string                      string_dequeue_to_end   ();
    size_t                      length_to_end           ();


  private:
    Byte*                       request_write_buffer_2  ( int size, int used_length );


    struct Buffer
    {
                                Buffer                  ()                                          : _ptr(NULL), _size(0) {}
                                Buffer                  ( const Buffer& )                           : _ptr(NULL), _size(0) {}
                               ~Buffer                  ();

        void                    allocate                ( int size );

        Byte*                  _ptr;
        int                    _size;
    };


    Fill_zero                  _zero_;
    size_t                     _write_position;
    int const                  _chunk_size;

    typedef std::list<Buffer>   Buffer_list;
    Buffer_list                _buffer_list;

    Buffer_list::iterator      _read_iterator;
    size_t                     _read_position;

  public:
    static const int            default_chunk_size;
};

//------------------------------------------------------------------------------------Gzip_deflator

struct Gzip;

struct Gzip_deflator : Object
{
                                Gzip_deflator           ( Simple_byte_queue* );
                               ~Gzip_deflator           ();

    void                        write                   ( const Byte*, int length );

  private:
    Fill_zero                  _zero_;
    ptr<Gzip>                  _gzip;
};

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer

#endif
